<?php
/*
 * Plugin Name: amarpaybd
 * Plugin URI: https://wordpress.org/plugins/amarpaybd
 * Description: This plugin allows your customers to pay with Bkash, Nagad, Rocket, and all BD gateways via amarpaybd.
 * Author: BlitheForge
 * Author URI: https://github.com/
 * Version: 1.0.1
 * Requires at least: 5.2
 * Requires PHP: 7.2
 * License: GPL v2 or later
 * License URI: 
 * Text Domain: amarpaybd
 */

if (!defined('ABSPATH')) {
    exit;
}

// Ensure WooCommerce is loaded before initializing
add_action('plugins_loaded', 'amarpaybd_init_gateway_class');

function amarpaybd_init_gateway_class()
{
    if (!class_exists('WC_Payment_Gateway')) return;

    class WC_Amarpaybd_Gateway extends WC_Payment_Gateway
    {
        public function __construct()
        {
            $this->id = 'amarpaybd';
            $this->icon = 'https://amarpaybd.com/amarpaybd.png';
            $this->has_fields = false;
            $this->method_title = __('amarpaybd', 'amarpaybd');
            $this->method_description = __('Pay With amarpaybd', 'amarpaybd');

            $this->supports = array('products');

            $this->init_form_fields();
            $this->init_settings();

            $this->title = $this->get_option('title');
            $this->description = $this->get_option('description');
            $this->enabled = $this->get_option('enabled');

            add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));
            add_action('woocommerce_api_' . strtolower(get_class($this)), array($this, 'handle_webhook'));
        }

        public function init_form_fields()
        {
            $this->form_fields = array(
                'enabled' => array(
                    'title'       => 'Enable/Disable',
                    'label'       => 'Enable amarpaybd',
                    'type'        => 'checkbox',
                    'description' => '',
                    'default'     => 'no'
                ),
                'title' => array(
                    'title'       => 'Title',
                    'type'        => 'text',
                    'description' => 'This controls the title which the user sees during checkout.',
                    'default'     => 'amarpaybd Gateway',
                    'desc_tip'    => true,
                ),
                'apikeys' => array(
                    'title'       => 'Enter API Key',
                    'type'        => 'text',
                    'description' => '',
                    'default'     => '',
                    'desc_tip'    => true,
                ),
                'currency_rate' => array(
                    'title'       => 'Enter USD Rate',
                    'type'        => 'number',
                    'description' => '',
                    'default'     => '',
                    'desc_tip'    => true,
                ),
                'is_digital' => array(
                    'title'       => 'Enable/Disable Digital product',
                    'label'       => 'Enable Digital product',
                    'type'        => 'checkbox',
                    'description' => '',
                    'default'     => 'no'
                ),
                'payment_site' => array(
                    'title'       => 'Payment Site URL',
                    'type'        => 'text',
                    'description' => '',
                    'default'     => 'https://pay.amarpaybd.com/',
                    'desc_tip'    => true,
                    'custom_attributes' => array(
                        'readonly' => 'readonly'
                    ),
                ),
            );
        }

        public function process_payment($order_id)
        {
            $order = wc_get_order($order_id);
            $current_user = wp_get_current_user();

            $total = $order->get_total();
            if ($order->get_currency() == 'USD') {
                $total *= floatval($this->get_option('currency_rate'));
            }

            $data = array(
                "cus_name"    => $current_user->user_firstname,
                "cus_email"   => $current_user->user_email,
                "amount"      => $total,
                "webhook_url" => site_url('/?wc-api=wc_amarpaybd_gateway&order_id=' . $order->get_id()),
                "success_url" => $this->get_return_url($order),
                "cancel_url"  => wc_get_checkout_url()
            );

            $header = array(
                "api" => $this->get_option('apikeys'),
                "url" => $this->get_option('payment_site') . "api/payment/create"
            );

            $response = $this->create_payment($data, $header);
            $data = json_decode($response, true);

            if (isset($data['payment_url'])) {
                return array(
                    'result'   => 'success',
                    'redirect' => $data['payment_url']
                );
            } else {
                wc_add_notice(__('Payment gateway error: Unable to create payment.', 'amarpaybd'), 'error');
                return array('result' => 'fail');
            }
        }

        public function create_payment($data, $header)
        {
            $headers = array(
                'Content-Type: application/json',
                'API-KEY: ' . $header['api'],
            );
            $url = $header['url'];
            $data_json = json_encode($data);

            $curl = curl_init();
            curl_setopt_array($curl, array(
                CURLOPT_URL => $url,
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_POST => true,
                CURLOPT_POSTFIELDS => $data_json,
                CURLOPT_HTTPHEADER => $headers,
                CURLOPT_SSL_VERIFYPEER => false,
            ));

            $response = curl_exec($curl);
            curl_close($curl);
            return $response;
        }

        public function handle_webhook()
        {
            $order_id = $_GET['order_id'];
            $transactionId = $_REQUEST['transactionId'];

            if (!$order_id || !$transactionId) {
                error_log("Webhook Error: Missing order_id or transactionId.");
                exit();
            }

            $order = wc_get_order($order_id);
            if (!$order) {
                error_log("Webhook Error: Order not found.");
                exit();
            }

            $data = array("transaction_id" => $transactionId);
            $header = array(
                "api" => $this->get_option('apikeys'),
                "url" => $this->get_option('payment_site') . "api/payment/verify"
            );

            $response = $this->create_payment($data, $header);
            $data = json_decode($response, true);

            if ($data['status'] == "COMPLETED") {
                $order->payment_complete();
                $order->update_status('completed', __('Payment confirmed via webhook.', 'amarpaybd'));
            } else {
                $order->update_status('failed', __('Payment failed via webhook.', 'amarpaybd'));
            }

            status_header(200);
            echo json_encode(['message' => 'Webhook processed successfully.']);
            exit();
        }
    }

    function amarpaybd_add_gateway_class($gateways)
    {
        $gateways[] = 'WC_Amarpaybd_Gateway';
        return $gateways;
    }
    add_filter('woocommerce_payment_gateways', 'amarpaybd_add_gateway_class');
}

add_action('rest_api_init', function () {
    register_rest_route('Amarpaybd/v1', '/webhook', array(
        'methods' => 'POST',
        'callback' => array(new WC_Amarpaybd_Gateway(), 'handle_webhook'),
    ));
});
